@@ -3,9 +3,10 @@ package checks
 import (
 	"context"
 	"net"
-	"time"
 
 	"github.com/pkg/errors"
+
+	gosundheit "github.com/AppsFlyer/go-sundheit"
 )
 
 // Pinger verifies a resource is still alive.
@@ -22,18 +23,16 @@ func (f PingContextFunc) PingContext(ctx context.Context) error {
 	return f(ctx)
 }
 
-// NewPingCheck returns a Check that pings using the specified Pinger and fails on timeout or ping failure
-func NewPingCheck(name string, pinger Pinger, timeout time.Duration) (Check, error) {
+// NewPingCheck returns a Check that pings using the specified Pinger and fails on context cancellation or ping failure
+func NewPingCheck(name string, pinger Pinger) (gosundheit.Check, error) {
 	if pinger == nil {
 		return nil, errors.New("Pinger must not be nil")
 	}
 
 	return &CustomCheck{
 		CheckName: name,
-		CheckFunc: func() (details interface{}, err error) {
-			pingCtx, cancel := context.WithTimeout(context.Background(), timeout)
-			defer cancel()
-			return nil, pinger.PingContext(pingCtx)
+		CheckFunc: func(ctx context.Context) (details interface{}, err error) {
+			return nil, pinger.PingContext(ctx)
 		},
 	}, nil
 }
